
filename = 'APPLE'
offset_start = 0        
offset_end = 50          
step   = 75                                         
radius = 10 
         
level_1 = 0.9
level_2 = 0.95     
level_3 = 0.9
level_4a = 0.9
level_4b = 0.6

#--------------------------------------------------------------------------------
#--------------------------------------------------------------------------------
import matplotlib.pyplot as plt                                 # If you don't have this library, run pip install matplotlib from the Windows command line.
import numpy as np                                              # If you don't have this library, run pip install numpy from the Windows command line.
import random

#-----------------------------Mean value function--------------------------------
def average(signal):                                            
    window_size = 10                                            
    half_window = window_size // 2                              
    signal_smoothed = signal.copy()                             
    for i in range(half_window, len(signal) - half_window):
        window = signal[i - half_window:i + half_window + 1]    
        signal_smoothed[i] = sum(window) / window_size
    return signal_smoothed
#--------------------------------------------------------------------------------

#-----------------------------Drawing solid circles------------------------------
def draw_dot(dot_x,dot_y,dot_detected):
    pos_x = (dot_x+1) *75+ offset_start
    pos_y = dot_y *(-1000)+500
    color_inner = 'red'
    color_line = 'lightgray'
    r = 5                                                      
    theta = np.linspace(0, 2*np.pi, 40)                         
    x = r * np.cos(theta)
    y = r * np.sin(theta)*40                                    
    if(dot_detected):plt.plot(x+pos_x, y+pos_y, linewidth = 2*r ,color=color_inner)  
    plt.plot(x*2+pos_x, y*2+pos_y, linewidth = r/2 ,color=color_line)    
#--------------------------------------------------------------------------------
    
#-----------------------------Braille to characters------------------------------
def Dot2Char(dota,dotb):  
    if ( dota==2 and dotb==1 ):  return "A"
    if ( dota==5 and dotb==1 ):  return "B"
    if ( dota==2 and dotb==2 ):  return "C"
    if ( dota==2 and dotb==5 ):  return "D"
    if ( dota==2 and dotb==3 ):  return "E"
    if ( dota==5 and dotb==2 ):  return "F"
    if ( dota==5 and dotb==5 ):  return "G"
    if ( dota==5 and dotb==3 ):  return "H"
    if ( dota==3 and dotb==2 ):  return "I"
    if ( dota==3 and dotb==5 ):  return "J"
    if ( dota==6 and dotb==1 ):  return "K"
    if ( dota==8 and dotb==1 ):  return "L"
    if ( dota==6 and dotb==2 ):  return "M"
    if ( dota==6 and dotb==5 ):  return "N"
    if ( dota==6 and dotb==3 ):  return "O"
    if ( dota==8 and dotb==2 ):  return "P"
    if ( dota==8 and dotb==5 ):  return "Q"
    if ( dota==8 and dotb==3 ):  return "R"
    if ( dota==7 and dotb==2 ):  return "S"
    if ( dota==7 and dotb==5 ):  return "T"
    if ( dota==6 and dotb==4 ):  return "U"
    if ( dota==8 and dotb==4 ):  return "V"
    if ( dota==3 and dotb==8 ):  return "W"
    if ( dota==6 and dotb==6 ):  return "X"
    if ( dota==6 and dotb==8 ):  return "Y"
    if ( dota==6 and dotb==7 ):  return "Z"
    else: return "?"
#--------------------------------------------------------------------------------

#-----------------------------Main Program---------------------------------------

# 1.Read files.
file =open( filename + '.txt')            
full_txt = file.readlines()                                     
total_lines = len(full_txt) - offset_end                        
char_num = int( total_lines/150 )                               
file.close()                                                    

# 2.Construct one-dimensional arrays (floating point numbers).
data1_original = 1.0 * np.arange(total_lines)                   
data2_original = 1.0 * np.arange(total_lines)
data3_original = 1.0 * np.arange(total_lines)
data4_original = 1.0 * np.arange(total_lines) 

# 3.Read data into the array.
for i in range(total_lines):
    data1_original[i] = full_txt[i].split()[2]                      
    data2_original[i] = full_txt[i].split()[3]                   
    data3_original[i] = full_txt[i].split()[4]                  
    data4_original[i] = full_txt[i].split()[5]                  

# 4.Mean filtering.
data1 = average(data1_original)                             
data2 = average(data2_original)    
data3 = average(data3_original)
data4 = average(data4_original)

# 5.Maximum value.
max1= np.max(data1)                                         
max2= np.max(data2) 
max3= np.max(data3) 
max4= np.max(data4)
    
# 6.Plot waveforms.    
fig = plt.figure(figsize=(20, 10))                              
plt.plot(data1_original+6000, color ='lightgray')               
plt.plot(data2_original+5000, color ='lightgray')  
plt.plot(data3_original+4000, color ='lightgray') 
plt.plot(data4_original+3000, color ='lightgray')

plt.plot(data1+6000, color ='blue')  
plt.plot(data2+5000, color ='red')  
plt.plot(data3+4000, color ='yellow') 
plt.plot(data4+3000, color ='green')

# 7.Construct 4 arrays to store test results(initial value 0).
dot1 = 0 * np.arange(2*char_num)                                
dot2 = 0 * np.arange(2*char_num)
dot3 = 0 * np.arange(2*char_num)
dot4 = 0 * np.arange(2*char_num)    
    
# 8.Detection.
for i in range(2*char_num):
    pos = offset_start + (i+1)*step - radius
    for j in range(2*radius):
        if (data1[pos+j] < max1 * level_1): dot1[i]=1
        
        if (data2[pos+j] < max2 * level_2): dot2[i]=1
            
        if (data3[pos+j] < max3 * level_3): dot3[i]=1
            
        if (data4[pos+j] < max4 * level_4a): dot4[i]=1
        if (data4[pos+j] < max4 * level_4b): dot4[i]=2        

# 9.Display test results.
for i in range(char_num*2):       
    plt.text( (i+1) *75+ offset_start, 7500, dot1[i], fontdict={'family': 'serif', 'size': 20, 'color': 'blue'}, ha='center') 
    plt.text( (i+1) *75+ offset_start, 6100, dot2[i], fontdict={'family': 'serif', 'size': 20, 'color': 'red'}, ha='center') 
    plt.text( (i+1) *75+ offset_start, 4500, dot3[i], fontdict={'family': 'serif', 'size': 20, 'color': 'yellow'}, ha='center') 
    plt.text( (i+1) *75+ offset_start, 3200, dot4[i], fontdict={'family': 'serif', 'size': 20, 'color': 'green'}, ha='center') 

# 10.Determine the corresponding 8 states according to the dot array.
state = 0 * np.arange(2*char_num)
for i in range(2*char_num):
    if( dot1[i]==0 and dot2[i]==0 and dot3[i]==0 and dot4[i]==0 ): state[i]=1
    if( dot1[i]==0 and dot2[i] >0 and dot3[i]==0 and dot4[i] >0 ): state[i]=2
    if( dot1[i] >0 and dot2[i]==0 and dot3[i]==0 and dot4[i] >0 ): state[i]=3
    if( dot1[i] >0 and dot2[i]==0 and dot3[i] >0 and dot4[i]==0 ): state[i]=4
    if( dot1[i] >0 and dot2[i] >0 and dot3[i]==0 and dot4[i] >0 ): state[i]=5
    if( dot1[i] >0 and dot2[i] >0 and dot3[i] >0 and dot4[i] >0 ): state[i]=6
    if( dot1[i] >0 and dot2[i]==0 and dot3[i] >0 and dot4[i] >0 ): state[i]=7
    if( dot1[i] >0 and dot2[i] >0 and dot3[i] >0 and dot4[i] >1 ): state[i]=8
    plt.text( (i+1) *75+ offset_start, 1000, state[i], fontdict={'family': 'serif', 'size': 20, 'color': 'black'}, ha='center') 
    
# 11.Draw points based on the state array.
for i in range(char_num*2):     
    if ( state[i]==1 ) :   
        draw_dot(i,0,0)
        draw_dot(i,1,0)
        draw_dot(i,2,0)
    if ( state[i]==2 ) :   
        draw_dot(i,0,1)
        draw_dot(i,1,0)
        draw_dot(i,2,0)
    if ( state[i]==3 ) :   
        draw_dot(i,0,0)
        draw_dot(i,1,1)
        draw_dot(i,2,0)
    if ( state[i]==4 ) :   
        draw_dot(i,0,0)
        draw_dot(i,1,0)
        draw_dot(i,2,1)
    if ( state[i]==5 ) :   
        draw_dot(i,0,1)
        draw_dot(i,1,1)
        draw_dot(i,2,0)
    if ( state[i]==6 ) :   
        draw_dot(i,0,1)
        draw_dot(i,1,0)
        draw_dot(i,2,1)
    if ( state[i]==7 ) :   
        draw_dot(i,0,0)
        draw_dot(i,1,1)
        draw_dot(i,2,1)
    if ( state[i]==8 ) :   
        draw_dot(i,0,1)
        draw_dot(i,1,1)
        draw_dot(i,2,1)

# 12.Call the function Dot2Char() to parse the characters according to the state, two by two.
result =''
for i in range(char_num):
    result += Dot2Char(state[2*i],state[2*i+1])
    
# 13.Display characters.
plt.plot(0, -3500)                              
for i in range(char_num):       
    plt.text( (i+1) *150 -38 + offset_start, -3500, result[i], fontdict={'family': 'serif', 'size': 40, 'color': 'black'}, ha='center')      

# 14.Show chart.
plt.axis('off')                      
plt.show()
